# Эмулятор для разработки приложений

Предназначен для разработки приложений без установки на робота, представляет собой страницу iframe-ом в который загружается приложение.
Обмен данными осуществляется по ws с помощью библиотеки socket.io.

## Как работают приложения на роботе

QtWebEngine в GUI робота представляет из себя браузер Chrome 56, версия может меняться от релиза к релизу. Само веб приложение складывается на робота в папку, которая будет доступна по адресу
http://localhost/[имя проекта]/

На главной странице приложения можно найти такие строки

```
<script src="qrc:///qtwebchannel/qwebchannel.js"></script>
<script src="js/api/lib/logger.js"></script>
<script src="js/api/lib/promobot.api-1.7.1.js"></script>
```

* qwebchannel.js - предназначен для обмена сообщениями с роботом по ws
* logger.js - библиотека для логирования
* promobot.api-1.7.1.js - библиотека API робота

С помощью библиотеки можно или подписаться на нужные события или отправить роботу сообщение на выполнение команды: произнести текст, поехать на точку и т.д.

При этом если в веб приложении настроено логирование сообщений с помощью logger.js то можно получать сообщения из приложения о том какая команда пришла в приложение и что оно при этом начало выполнять. Библиотека логирования тоже работает на основе socket.io.

Этот обмен сообщениями можно эмулировать на компьютере разработчика.

## Запуск эмулятора

в файле /public/js/applications.js перечисляем список приложений и их локальных url

```
let applications = [
  {
    name: 'sample1',
    url: 'http://localhost:8080'
  },
  {
    name: 'sample2',
    url: 'http://localhost:8080'
  }
]
```

стартуем эмулятор и открываем http://localhost:8089/

```
npm start
```

порт 8089 можно сменить в файле ```server.js```

## Backend для приложения

Приложение робота представляет из себя статичное SPA. При необходимости backend части приложения на роботе оно оформляется отдельно и с помощью docker контейнера запускается на роботе. Обмен между SPA и backend частью можно осуществить по ajax или ws. Backend приложение пишется на языках Python, NodeJs, PHP.

## Разработка приложения

для определения в каком окружении находится приложение используем переменную QWebChannel

```
var emulatorUrl = 'http://localhost:8089'
const env = (typeof QWebChannel !== 'undefined') ? 'production' : 'development'
const robotInstance = (env === 'development') ? 'getEmulatorInstance' : 'getInstance'
```

метод getEmulatorInstance ожидает глобальную переменную emulatorUrl для подключения к эмулятору

так же желательно подключить логирование на этот же url

```
const logger = new StatLogger()
logger.setRobotAppId('org.promobot.example.jsapp')
logger.setRobotAppLang('570')
logger.setRobotAppVer('0.0.1')
logger.setRobotUuid('237593f1-dd1b-4b1b-b8d7-741665db64e5')
logger.addConnection({ url : emulatorUrl, reconnection : false })
```

подключаемся к API (JS версия)
```
document.addEventListener('DOMContentLoaded', () => {
  Promobot[robotInstance]().then(promobot => {
    promobot.dialogService.sayText('Привет мир')
  })
})
```

подключаемся к API (ES6 версия)
```
settings = {
  emulator: {
    url: 'http://localhost:8089',
    reconnection: false,
    secure: false
  }
}

const environment = (typeof QWebChannel !== 'undefined') ? 'production' : 'development'
const robotInstance = (environment === 'development') ? Promobot.getEmulateInstance(settings.emulator) : Promobot.getInstance()
robotInstance.then(promobot => {
  promobot.dialogService.sayText('Привет мир')
})
```

## Процесс разработки

Используя в приложении вместо api робота эмулятор и направив логирование тоже в него, мы можем сэмулировать отправку события от робота и увидеть в логировании получение этого события и реакцию на него.

Большая часть процесса разработки приложения состоит из обработки получамых сообщений и реакции на них.

В процессе разработки мы пришли к следующим выводам:

* Необходимо использовать какую либо библиотеку для рендера страниц (React, Vue)
* Удобнее если весь обмен сообщениями с API производить через единый центр
* Удобнее хранить состояние приложение в едином хранилище

Под эти цели достаточно хорошо подходил фреймворк Vue, пример его использования и есть в js-example.

## Логирование

Для сохрания статистики работы приложения была разработана библиотека логирования событий, действий, состояний робота, ее подключение есть в примере.

Для того чтобы события стали отправляться необходимо заполнить обязательные поля (в процеессе отладки можно взять отсюда):

* идентификатор приложения - создается при регистрации приложения в БД
* язык приложения - создается при регистрации приложения в БД
* версия приложения - устанавливается разработчиком
* id робота - на самом роботе надо взять из окружения

и добавить подключение, в процессе отладки указать эмулятор

```
logger.setRobotAppId('org.promobot.example.jsapp') // id приложения
logger.setRobotAppLang('570') // id языка приложения (570 - русский)
logger.setRobotAppVer('0.0.1') // версия приложения
logger.setRobotUuid('237593f1-dd1b-4b1b-b8d7-741665db64e5') // id робота
logger.addConnection({ url : emulatorUrl, reconnection : false }) // коннект
```

логируем входящие события - ``EVENT``

```
promobot.userService.onUserPresence((user, isNew) => {
  let eventId = logger.logEvent( EventInitiatorTypes.ROBOT, EventTypes.USER_PRESENCE, { user : user })
})

promobot.environmentService.get().then(environment => {
  let eventId = logger.logEvent( EventInitiatorTypes.ROBOT, EventTypes.ROBOT_LOAD_ENVIRONMENT, environment )
})
```

логируем команды приложения - ``ACTION`` и выполняем их, при логировании действия указываем eventId - id события которое вызвало его

```
promobot.environmentService.get().then(environment => {
  let eventId = logger.logEvent( EventInitiatorTypes.ROBOT, EventTypes.ROBOT_LOAD_ENVIRONMENT, environment )
  let text = 'я получил данные окружения'
  logger.logAction(eventId, ActionHandlerTypes.ROBOT, ActionTypes.ROBOT_SAY_TEXT, {text: text})
  robot.dialogService.sayText(text)
})
```

логируем состояние приложения - ``STATE`` при логировании состояния указываем eventId - id события которое привело приложение в него

```
logger.logState(eventId, {
  stateName: 'SERVICE-CHOICE'
})
```

наименования состояний составляются при разработке приложения
