const https = require('https')
let express = require('express')
let app = express()
let fs = require('fs')
let moment = require('moment')

let privateKey = fs.readFileSync('./ssl/cert.key')
let certificate = fs.readFileSync('./ssl/cert.crt')

let server = null
let useSecure = false
let port = 8096

if (useSecure) {
  server = https.createServer({
      key: privateKey,
      cert: certificate,
      requestCert: false,
      rejectUnauthorized: false
  }, app).listen(port)
} else {
  server = require('http').Server(app).listen(port)
}

app.use(function(req, res, next) {
  res.header("Access-Control-Allow-Origin", "*")
  res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept")
  next()
})

let routes = [
  {
    route: '/departments',
    response: {
      status: 'success',
      data: [
        {id: 1, name: 'Административно-управленческий персонал'},
        {id: 2, name: 'Служба безопасности'},
        {id: 3, name: 'Юридическая служба'},
        {id: 4, name: 'Бухгалтерия'},
        {id: 5, name: 'Отдел делопроизводства'},
        {id: 6, name: 'Отдел кадастровых инженеров'}
      ]
    }
  },
  {
    route: '/employees',
    response: {
      status: 'success',
      data: [
        {id: 1, department_id: 1, name: 'Лобанув Николай Анатольевич'},
        {id: 2, department_id: 2, name: 'Лобанов Николай Анатольевич'},
        {id: 3, department_id: 3, name: 'Лобаныв Николай Анатольевич'},
        {id: 4, department_id: 4, name: 'Лобанев Николай Анатольевич'},
        {id: 5, department_id: 5, name: 'Лобанив Николай Анатольевич'},
        {id: 6, department_id: 6, name: 'Лобаняв Николай Анатольевич.'}
      ]
    }
  }
]

app.get('/', function (req, res) {
  res.json(routes)
})

app.get('*', function (req, res) {
  let data = routes.find(r => { return r.route === req.path })
  if (data) {
    res.json(data.response)
  } else {
    res.json({
      status: 'error'
    })
  }
})
