const https = require('https')
let express = require('express')
let app = express()
let fs = require('fs')
let moment = require('moment')

let privateKey = fs.readFileSync('./ssl/cert.key')
let certificate = fs.readFileSync('./ssl/cert.crt')

let server = null
let useSecure = false
let port = 8096

if (useSecure) {
  server = https.createServer({
      key: privateKey,
      cert: certificate,
      requestCert: false,
      rejectUnauthorized: false
  }, app).listen(port)
} else {
  server = require('http').Server(app).listen(port)
}

app.use(function(req, res, next) {
  res.header("Access-Control-Allow-Origin", "*")
  res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept")
  next()
})

let zones = [
  { Slot0: 'Empty' },
  { Slot1: 'Empty' },
  { Slot2: 'Empty' },
  { Slot3: 'Empty' },
  { Slot4: 'Empty' },
  { Slot5: 'Empty' },
  { Slot6: 'Empty' },
  { Slot7: 'Empty' },
  { Slot8: 'Empty' },
  { Slot9: 'Empty' }
]

let states = {
  faces: 'end',
  elevator: 'end',
  mbytt: 'end'
}

app.get('/ResetZones.php', function (req, res) {
  zones = [
    { Slot0: 'Empty' },
    { Slot1: 'Empty' },
    { Slot2: 'Empty' },
    { Slot3: 'Empty' },
    { Slot4: 'Empty' },
    { Slot5: 'Empty' },
    { Slot6: 'Empty' },
    { Slot7: 'Empty' },
    { Slot8: 'Empty' },
    { Slot9: 'Empty' }
  ]
  res.json({
    Answer: null,
    Error: null
  })
})

app.get('/GetZones.php', function (req, res) {
  res.json({
    Answer: zones,
    Error: null
  })
})

app.get('/StartMoving.php', function (req, res) {
  zones.find(z => z[req.query.startPoint])[req.query.startPoint] = 'MovingFrom'
  zones.find(z => z[req.query.endPoint])[req.query.endPoint] = 'MovingTo'
  res.json({
    Answer: null,
    Error: null
  })
})

app.get('/EndMoving.php', function (req, res) {
  zones.find(z => z[req.query.startPoint])[req.query.startPoint] = 'Empty'
  zones.find(z => z[req.query.endPoint])[req.query.endPoint] = 'Locked'
  res.json({
    Answer: null,
    Error: null
  })
})

app.get('/Elevator.php', function (req, res) {
  let answer = null
  if (req.query.roboId) {
    states.elevator = 'playing'
    setTimeout(() => {
      states.elevator = 'end'
    }, 10000)
  } else {
    answer = {
      state: states.elevator
    }
  }
  res.json({
    Answer: answer,
    Error: null
  })
})

app.get('/MBYTT.php', function (req, res) {
  let answer = null
  if (req.query.roboId) {
    states.mbytt = 'playing'
    setTimeout(() => {
      states.mbytt = 'end'
    }, 10000)
  } else {
    answer = {
      state: states.mbytt
    }
  }
  res.json({
    Answer: answer,
    Error: null
  })
})

app.get('/Faces.php', function (req, res) {
  let answer = null
  if (req.query.roboId) {
    states.faces = 'playing'
    setTimeout(() => {
      states.faces = 'end'
    }, 20 * 1000)
  } else {
    answer = {
      state: states.faces
    }
  }
  res.json({
    Answer: answer,
    Error: null
  })
})
