const emulatorOptions = {
  url: window.location.protocol + '//' + window.location.hostname + ':8089',
  secure: (window.location.protocol === 'http:')
}

const neuro = [
  {classification: {category: "справканесудимость_заход", question_level: "услуга 2.2", question_type: "основное"}, similar: "Какие документы нужны для замены паспорта", sure_level: 0.9},
  {classification: {category: "паспорт_документы", question_level: "услуга 2.2", question_type: "основное"}, similar: "Какие документы нужны для замены паспорта", sure_level: 0.76},
  {classification: {category: "загран_документы", question_level: "услуга 5.2", question_type: "основное"}, similar: "Какие документы нужны для замены паспорта.", sure_level: 0.76},
  {classification: {category: "паспорт_документы", question_level: "услуга 2.2", question_type: "основное"}, similar: "какие документы нужны для нового паспорта?", sure_level: 0.5}
]

const neuro_2 = [
  {classification: {category: "перерасчетпенсии_заход", question_level: "услуга 2.2", question_type: "основное"}, similar: "Какие документы нужны для замены паспорта", sure_level: 0.9},
  {classification: {category: "паспорт_документы", question_level: "услуга 2.2", question_type: "основное"}, similar: "Какие документы нужны для замены паспорта", sure_level: 0.76},
  {classification: {category: "загран_документы", question_level: "услуга 5.2", question_type: "основное"}, similar: "Какие документы нужны для замены паспорта.", sure_level: 0.76},
  {classification: {category: "паспорт_документы", question_level: "услуга 2.2", question_type: "основное"}, similar: "какие документы нужны для нового паспорта?", sure_level: 0.5}
]

const store = new Vuex.Store({
  state: {
    applications:       applications,
    application:        null,
    url:                null,
    debug:              true,
    case_sessions:      [],
    intr_sessions:      [],
    events:             [],
    actions:            [],
    states:             [],
    messages:           [],
    robots:   [
      {type: 'promobotv4_0058', label: 'promobotv4_0058'},
      {type: 'promobotv4_0059', label: 'promobotv4_0059'},
      {type: 'promobotv4_0071', label: 'promobotv4_0071'}
    ],
    printer_statuses:   [
      {type: 'IDLE', label: 'пауза'},
      {type: 'LOW-PAPER', label: 'мало бумаги'},
      {type: 'ERROR', label: 'ошибка'},
      {type: 'SUCCESS', label: 'успешно'}
    ],
    dispenser_statuses: [
      {type: 'NOT-CONNECTED', label: 'не присоединен'},
      {type: 'EMPTY', label: 'пустой'},
      {type: 'SUCCESS', label: 'успешно'}
    ],
    drive_statuses: [
      {type: 'OBSTACLE', label: 'препятствие'},
      {type: 'STUCKED', label: 'застрял'},
      {type: 'SUCCESS', label: 'успешно'}
    ],
    scanner_statuses: [
      {type: 'ERROR', label: 'не успешно'},
      {type: 'SUCCESS', label: 'успешно'}
    ],
    emulator: {
      printer_status: null,
      dispenser_status: null,
      drive_status: null,
      scanner_status: null
    }
  },
  actions: {
    changeApplication: ({store, dispatch}, app) => {
      dispatch('changeApplicationUrl', app.url)
    },
    changePrinterStatus: ({store, dispatch}, status) => {
      socket.emit('emulator-state', {meta: 'PRINTER-STATE', data: status.type})
    },
    changeDispenserStatus: ({store, dispatch}, status) => {
      socket.emit('emulator-state', {meta: 'DISPENSER-STATE', data: status.type})
    },
    changeDriveStatus: ({store, dispatch}, status) => {
      socket.emit('emulator-state', {meta: 'DRIVE-STATE', data: status.type})
    },
    changeScannerStatus: ({store, dispatch}, status) => {
      socket.emit('emulator-state', {meta: 'SCANNER-STATE', data: status.type})
    },
    changeApplicationDebug: ({store, dispatch}, status) => {
      socket.emit('emulator-state', {meta: 'DEBUG', data: status.type})
    },
    changeApplicationUrl: (store, url) => store.commit('CHANGE_APPLICATION_URL', url),
    clearMessages: (store) => store.commit('CLEAR_MESSAGES'),
    updateMessage: (store, data) => store.commit('UPDATE_MESSAGE', data),
  },
  mutations: {
    CHANGE_PRINTER_STATUS: (state, status) => state.emulator.printer_status = status,
    CHANGE_DISPENSER_STATUS: (state, status) => state.emulator.dispenser_status = status,
    CHANGE_DRIVE_STATUS: (state, status) => state.emulator.drive_status = status,
    CHANGE_APPLICATION: (state, app) => state.application = app,
    CHANGE_APPLICATION_URL: (state, url) => state.url = url,
    ADD_INTR_SESSION: (state, data) => state.intr_sessions.push(data),
    ADD_CASE_SESSION: (state, data) => state.case_sessions.push(data),
    ADD_EVENT: (state, data) => state.events.push(data),
    ADD_ACTION: (state, data) => state.actions.push(data),
    ADD_STATE: (state, data) => state.states.push(data),
    ADD_MESSAGE: (state, data) => state.messages.push(data),
    UPDATE_MESSAGE: (state, data) => state.messages.splice(data.index, 1, data.message),
    CLEAR_MESSAGES: (state) => state.messages = [],
  },
  getters: {
    applications: (state) => state.applications,
    application: (state) => state.application,
    emulator: (state) => state.emulator,
    robots: (state) => state.robots,
    printerStatus: (state) => state.emulator.printer_status,
    printerStatuses: (state) => state.printer_statuses,
    dispenserStatus: (state) => state.emulator.dispenser_status,
    dispenserStatuses: (state) => state.dispenser_statuses,
    driveStatus: (state) => state.emulator.drive_status,
    driveStatuses: (state) => state.drive_statuses,
    scannerStatus: (state) => state.emulator.scanner_status,
    scannerStatuses: (state) => state.scanner_statuses,
    url: (state) => state.url,
    events: (state) => state.events,
    eventById: (state) => {
      return id => state.events.find(item => { return item.uuid === id })
    },
    actions: (state) => state.actions,
    states: (state) => state.states,
    messages: (state) => state.messages
  }
})

// logger
const socket = io(emulatorOptions.url, emulatorOptions)
socket.on('log', message => {
  if ( message.meta ) {
    // console.log(message.meta, message.data)
    // ...
    if (message.meta.object === 'EVENT' && message.meta.action === 'CREATE' && message.data.initiator === 'ROBOT' && message.data.type === 'ROBOT_REPLIC_START') {
      let msg = new SpeechSynthesisUtterance(message.data.data.text)
      let voices = window.speechSynthesis.getVoices()
      msg.voice = voices[15]
      msg.lang = 'ru-RU'
      // window.speechSynthesis.speak(msg)
    }
    // ...
    if ( message.meta.object === 'INTERACTION_SESSION' && message.meta.action === 'CREATE' ) {
      store.commit('ADD_INTR_SESSION', message.data)
      store.commit('ADD_MESSAGE', { type : 'interaction-session', id : message.data.uuid, text : 'CREATE: ' + ((message.data.data) ? JSON.stringify(message.data.data) : '') })
    }
    if ( message.meta.object === 'INTERACTION_SESSION' && message.meta.action === 'UPDATE' ) {
      store.commit('ADD_MESSAGE', { type : 'interaction-session', id : message.data.uuid, text : 'UPDATE: ' + ((message.data.data) ? JSON.stringify(message.data.data) : '') })
    }
    if ( message.meta.object === 'CASE_SESSION' && message.meta.action === 'CREATE' ) {
      store.commit('ADD_CASE_SESSION', message.data)
      store.commit('ADD_MESSAGE', { type : 'case-session', id : message.data.uuid, text : 'CREATE: ' + ((message.data.data) ? JSON.stringify(message.data.data) : '') })
    }
    if ( message.meta.object === 'CASE_SESSION' && message.meta.action === 'UPDATE' ) {
      store.commit('ADD_MESSAGE', { type : 'case-session', id : message.data.uuid, text : 'UPDATE: ' + ((message.data.data) ? JSON.stringify(message.data.data) : '') })
    }
    if ( message.meta.object === 'EVENT' && message.meta.action === 'CREATE' ) {
      store.commit('ADD_EVENT', message.data)
      store.commit('ADD_MESSAGE', { type : 'event', id : message.data.uuid, text : message.data.initiator + ' : ' + message.data.type + ' ' + ' : ' + message.data.uuid + ' ' + ' : ' + message.data.parentUuid + ' ' + ((message.data.data) ? JSON.stringify(message.data.data) : '') })
    }
    if ( message.meta.object === 'ACTION' && message.meta.action === 'CREATE' ) {
      store.commit('ADD_ACTION', message.data)
      store.commit('ADD_MESSAGE', { type : 'action', id : message.data.uuid, text : message.data.handler + ' : ' + message.data.type + ' ' + ((message.data.data) ? JSON.stringify(message.data.data) : '') })
    }
    if ( message.meta.object === 'STATE' && message.meta.action === 'CREATE' ) {
      store.commit('ADD_STATE', message.data)
      store.commit('ADD_MESSAGE', { type : 'state', id : message.data.uuid, text : message.data.name.toUpperCase() + ' ' + message.data.transition.toUpperCase() + ' ' + JSON.stringify(message.data.state) })
    }
  }
})

const iframe = Vue.component('Iframe', {
  computed: {
    url: function() { return this.$store.getters.url }
  },
  template: `<div id="iframe"><iframe v-bind:src="url" width="1280" height="765"></iframe></div>`
})

const menu = Vue.component('Menu', {
  data: function () {
    return {
      users: [
        { id: 1, id_track: 1, name: 'Ivan', general: false, active: false, age: 24, gender: 0, is_tracking: true, type: 2 },
        { id: 2, id_track: 2, name: 'Max', general: false, active: false, age: 24, gender: 0, is_tracking: true, type: 2 },
        { id: 3, id_track: 3, name: 'Anna', general: false, active: false, age: 24, gender: 0, is_tracking: true, type: 2 },
        { id: -1, id_track: 4, name: 'Ходырев', general: false, active: false, age: 35, gender: 0, is_tracking: true, type: -1 },
        { id: -2, id_track: 5, name: 'Шабалдас', general: false, active: false, age: 28, gender: 0, is_tracking: false, type: 2 }
      ]
    }
  },
  computed: {
    applications: function() { return this.$store.getters.applications },
    selectedApplication: {
      get: function() {
        return this.$store.getters.application
      },
      set: function(app) {
        this.$store.dispatch('changeApplication', app)
      }
    },

    robots: function() { return this.$store.getters.robots },
    robot: {
      get: function() {
        return this.$store.getters.robot
      },
      set: function(status) {
        this.$store.dispatch('changePrinterStatus', status)
      }
    },

    printerStatuses: function() { return this.$store.getters.printerStatuses },
    printerStatus: {
      get: function() {
        return this.$store.getters.printerStatus
      },
      set: function(status) {
        this.$store.dispatch('changePrinterStatus', status)
      }
    },
    dispenserStatuses: function() { return this.$store.getters.dispenserStatuses },
    dispenserStatus: {
      get: function() {
        return this.$store.getters.dispenserStatus
      },
      set: function(status) {
        this.$store.dispatch('changeDispenserStatus', status)
      }
    },
    driveStatuses: function() { return this.$store.getters.driveStatuses },
    driveStatus: {
      get: function() {
        return this.$store.getters.driveStatus
      },
      set: function(status) {
        this.$store.dispatch('changeDriveStatus', status)
      }
    },
    scannerStatuses: function() { return this.$store.getters.scannerStatuses },
    scannerStatus: {
      get: function() {
        return this.$store.getters.scannerStatus
      },
      set: function(status) {
        this.$store.dispatch('changeScannerStatus', status)
      }
    }
  },
  methods: {
    toggleUser: function (user) {
      user.active = !user.active
      if (user.active === false) {
        if (user.general === true) {
          socket.emit('emulator-command', {
            type: 'face.recognize.general',
            general: -2
          })
        }
        user.general = false
        socket.emit('emulator-state', {
          meta: 'REMOVE-FACE',
          data: user
        })
      } else {
        socket.emit('emulator-state', {
          meta: 'ADD-FACE',
          data: user
        })
      }
    },
    generalUser: function (user) {
      user.general = !user.general
      if (user.general === true) {
        if (user.active === false) {
          user.active = true
          socket.emit('emulator-state', {
            meta: 'ADD-FACE',
            data: user
          })
        }
        this.users.map(u => {
          if (u.id !== user.id) {
            u.general = false
          }
        })
        socket.emit('emulator-command', {
          type: 'face.recognize.general',
          general: user.id
        })
      } else {
        socket.emit('emulator-command', {
          type: 'face.recognize.general',
          general: -2
        })
      }
    },
    userHas: (id) => {
      switch (id) {
        case 1:
            socket.emit('emulator-command', { type: 'user.has.user', user_id: 1, last_visit: '2018-11-20 14:14:06', meet_date: '2018-11-20 13:47:47' })
          break;
        case 2:
            socket.emit('emulator-command', { type: 'user.has.user', user_id: 2, last_visit: '2018-11-20 14:14:06', meet_date: '' })
          break;
      }
    },
    settingsChange: () => socket.emit('emulator-command', {type: 'system.settings.update'}),
    userSay: () => socket.emit('emulator-command', {type: 'dialog.user.said', text: 'sample'}),
    robotSay: () => {
      socket.emit('emulator-command', {type: 'dialog.robot.replic.start', text: 'привет'})
      setTimeout(() => socket.emit('emulator-command', {type: 'dialog.robot.replic.finish'}), 1000)
    },
    userLost: () => socket.emit('emulator-command', {type: 'user.lost.user'}),
    interactionStart: () => socket.emit('emulator-command', {type: 'system.interaction.state', state: 'Interaction'}),
    interactionStop: () => socket.emit('emulator-command', {type: 'system.interaction.state', state: 'NonInteraction'}),
    caseStart: () => socket.emit('emulator-command', {type: 'system.custom.event.core', startCase: true}),
    caseStop: () => socket.emit('emulator-command', {type: 'system.custom.event.core', stopCase: true}),
    driveStart: () => socket.emit('emulator-command', {type: 'system.drive.state', start: true}),
    driveObstacle: () => socket.emit('emulator-command', {type: 'system.drive.state', obstacle: true}),
    driveStop: () => socket.emit('emulator-command', {type: 'system.drive.state', stop: true}),
    restart: () => socket.emit('emulator-command', {type: 'system.custom.event.core', restart: true}),
    operatorCallReplic: name => socket.emit('emulator-command', {type: 'dialog.action', meta: ['call-step-phrase', name, 'operator']}),
    operatorCallAction: action => socket.emit('emulator-command', {type: 'dialog.action', meta: action}),
    clearLog: () => store.dispatch('clearMessages'),
    changeApplication: (appName) => store.dispatch('changeApplication', appName),
    changeApplication: (name) => store.dispatch('changeApplication', name),
    emulateMessage: (payload) => socket.emit('emulator-command', payload),
    emulateMessages: messages => {
      if (Array.isArray(messages)) {
        messages.forEach(message => {
          socket.emit('emulator-command', message)
        })
      }
    },
    faceRecognize: data => {
      console.log('faceRecognize', data)
      // type - тип 0 = лицо пропало / 1 = лицо распознается / 3 = лицо распознано и есть во временной базе лиц / 2 = лицо распознано и есть в базе лиц
      let info = {
        face1: {
          age: 24,
          gender: 0,
          id: 1,
          id_track: 1,
          is_tracking: true,
          type: 2
        },
        face2: {
          age: 26,
          gender: 0,
          id: 2,
          id_track: 2,
          is_tracking: false,
          type: 2
        },
        face3: {
          age: 26,
          gender: 0,
          id: 3,
          id_track: 3,
          is_tracking: false,
          type: 2
        }
      }
      if (data.includes(1) === false) {
        info.face1.is_tracking = false
        info.face1.type = 0
      }
      if (data.includes(2) === false) {
        info.face2.is_tracking = false
        info.face2.type = 0
      }
      if (data.includes(3) === false) {
        info.face3.is_tracking = false
        info.face3.type = 0
      }
      socket.emit('emulator-command', {type: 'face.recognize.info', info: info})
    },
    setGeneral: id => {
      socket.emit('emulator-command', {type: 'face.recognize.general', general: id})
    },
    setGeneralSetted: id => {
      socket.emit('emulator-command', {type: 'system.general.setted', setted_id: id})
    }
  },
  template: `
  <div id="menu">
    <div>
      <button v-on:click="clearLog()">clear</button>
      <button v-on:click="restart()">restart</button>
      <select v-model="selectedApplication">
        <option>приложение</option>
        <option v-for="app in applications" v-bind:value="app">
          {{ app.name }}
        </option>
      </select>
      <select v-model="printerStatus">
        <option>принтер</option>
        <option v-for="status in printerStatuses" v-bind:value="status">
          {{ status.label }}
        </option>
      </select>
      <select v-model="dispenserStatus">
        <option>диспенсер</option>
        <option v-for="status in dispenserStatuses" v-bind:value="status">
          {{ status.label }}
        </option>
      </select>
      <select v-model="driveStatus">
        <option>движение</option>
        <option v-for="status in driveStatuses" v-bind:value="status">
          {{ status.label }}
        </option>
      </select>
      <select v-model="scannerStatus">
        <option>сканнер</option>
        <option v-for="status in scannerStatuses" v-bind:value="status">
          {{ status.label }}
        </option>
      </select>
    </div>
    <div>
      <button v-on:click="settingsChange()">settings</button>
      <button v-on:click="userHas(1)">user 1</button>
      <button v-on:click="userHas(2)">user 2</button>
      <button v-on:click="userLost()">user lost</button>
      <button v-on:click="emulateMessage({type: 'system.interaction.state', state: 'Interaction', Reason: 1})">int start 1</button>
      <button v-on:click="emulateMessage({type: 'system.interaction.state', state: 'Interaction', Reason: 2})">int start 2</button>
      <button v-on:click="interactionStop()">int stop</button>
      <button v-on:click="caseStart()">case start</button>
      <button v-on:click="caseStop()">case stop</button>

      <button v-for="user in users" v-bind:value="user.id" :class="{ active: user.active, general: user.general }">
        <span @click="toggleUser(user)">{{ user.name }}</span>
        <input type="checkbox" @change="generalUser(user)" :checked="user.general"/>
      </button>

      <button v-on:click="addFace(1)">add 1</button>
      <button v-on:click="addFace(2)">add 2</button>
      <button v-on:click="addFace(3)">add 3</button>
      <button v-on:click="removeFace(1)">del 1</button>
      <button v-on:click="removeFace(2)">del 2</button>
      <button v-on:click="removeFace(3)">del 3</button>

      <!--
      <button v-on:click="faceRecognize([])">fr 0</button>
      <button v-on:click="faceRecognize([1])">fr 1</button>
      <button v-on:click="faceRecognize([2])">fr 2</button>
      <button v-on:click="faceRecognize([3])">fr 3</button>
      <button v-on:click="faceRecognize([1,2])">fr 1,2</button>
      <button v-on:click="faceRecognize([2,3])">fr 2,3</button>
      -->
      <button v-on:click="setGeneral(1)">gen 1</button>
      <button v-on:click="setGeneral(2)">gen 2</button>
      <button v-on:click="setGeneral(3)">gen 3</button>
      <button v-on:click="setGeneral(-2)">gen -2</button>
      <button v-on:click="setGeneralSetted(1)">set 1</button>
      <button v-on:click="setGeneralSetted(2)">set 2</button>
    </div>
    <div>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: '123456', is_finish: true})">say 123456</button>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: 'Olga', is_finish: true})">say Olga</button>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: 'A175', is_finish: true})">say A175</button>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: 'проверить', is_finish: true})">say проверить</button>
      <button v-on:click="emulateMessage({type: 'dialog.greeting.load', greeting: {}})">say greeting</button>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: '7', is_finish: true})">say 7</button>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: '3:30', is_finish: true})">say 3:30</button>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: '7:00', is_finish: true})">say 7:00</button>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: '15:00', is_finish: true})">say 15:00</button>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: '1 число', is_finish: true})">say 1 число</button>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: '15 число', is_finish: true})">say 15 число</button>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: '20 число', is_finish: true})">say 20 число</button>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: 'Sed ut perspiciatis unde omnis iste natus error sit voluptatem', is_finish: true})">say l</button>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua', is_finish: true})">say xl</button>
      <button v-on:click="emulateMessage({type: 'dialog.user.said', text: 'Nemo enim ipsam voluptatem quia voluptas sit aspernatur aut odit aut fugit, sed quia consequuntur magni dolores eos qui ratione voluptatem sequi nesciunt. Neque porro quisquam est, qui dolorem ipsum quia dolor sit amet, consectetur, adipisci velit, sed quia non numquam eius modi tempora incidunt ut labore et dolore magnam aliquam quaerat voluptatem. Ut enim ad minima veniam, quis nostrum exercitationem ullam corporis suscipit laboriosam, nisi ut aliquid ex ea commodi consequatur? Quis autem vel eum iure reprehenderit qui in ea voluptate velit esse quam nihil molestiae consequatur, vel illum qui dolorem eum fugiat quo voluptas nulla pariatur?', is_finish: true})">say xxl</button>
      <button v-on:click="robotSay()">robot say</button>
      <button v-on:click="emulateMessage({type: 'dialog.question.new', text: 'Как настроение?', answers: ['Все ОК', 'Не очень']})">say question</button>
      <button v-on:click="emulateMessage({type: 'dialog.question.cancel'})">cancel question</button>
    </div>
    <div>
      <button v-on:click="emulateMessage({type: 'system.drive.mode', mode: 0})">drive manual</button>
      <button v-on:click="emulateMessage({type: 'system.drive.mode', mode: 1})">drive auto</button>

      <button v-on:click="emulateMessage({type: 'system.drive.status', status: 0})">drive idle</button>
      <button v-on:click="emulateMessage({type: 'system.drive.status', status: 1})">drive start</button>
      <button v-on:click="emulateMessage({type: 'system.drive.status', status: 2})">drive finish</button>
      <button v-on:click="emulateMessage({type: 'system.drive.status', status: 3})">drive cancel</button>
      <button v-on:click="emulateMessage({type: 'system.drive.status', status: 4})">drive obstacle</button>

      <button v-on:click="emulateMessage({type: 'system.drive.charge', charge: -1})">drive charge idle</button>
      <button v-on:click="emulateMessage({type: 'system.drive.charge', charge: 0})">drive charge ffm</button>
      <button v-on:click="emulateMessage({type: 'system.drive.charge', charge: 1})">drive charge scs</button>
      <button v-on:click="emulateMessage({type: 'system.drive.charge', charge: 2})">drive charge fcn</button>
      <button v-on:click="emulateMessages([{type: 'dialog.replica.extra.load', replica: {text: 'extra'}}])">extra</button>
      <button v-on:click="emulateMessage({type: 'dialog.question.cancel'})">clear q</button>
      <button v-on:click="emulateMessage({type: 'dialog.question.new', text: 'Вопрос 2', answers: ['Все ОК', 'Не очень']})">q2</button>
      <button v-on:click="emulateMessage({type: 'dialog.question.new', text: 'Вопрос 3', answers: ['Все ОК', 'Не очень', 'Хорошо']})">q3</button>
      <button v-on:click="emulateMessage({type: 'dialog.question.new', text: 'Вопрос 4', answers: ['Все ОК', 'Хорошо', 'Все ОК', 'Хорошо']})">q4</button>
      <button v-on:click="emulateMessage({type: 'dialog.question.new', text: 'Вопрос 6', answers: ['Все ОК', 'Хорошо', 'Все ОК', 'Хорошо', 'Все ОК', 'Хорошо']})">q6</button>
      <button v-on:click="emulateMessage({type: 'dialog.question.new', text: 'Вопрос 8', answers: ['Все ОК', 'Хорошо', 'Все ОК', 'Хорошо', 'Все ОК', 'Хорошо', 'Все ОК', 'Хорошо']})">q8</button>
      <button v-on:click="emulateMessage({type: 'dialog.question.new', text: 'Вопрос 10', answers: ['Все ОК', 'Хорошо', 'Все ОК', 'Хорошо', 'Все ОК', 'Хорошо', 'Все ОК', 'Хорошо', 'Все ОК', 'Хорошо']})">q10</button>
    </div>
    <div>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 1})">c1</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 2})">c2</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 3})">c3</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 4})">c4</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 5})">c5</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 6})">c6</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 7})">c7</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 8})">c8</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 9})">c9</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 10})">c10</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 11})">c11</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 12})">c12</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 13})">c13</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 14})">c14</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 15})">c15</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 16})">c16</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 17})">c17</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 18})">c18</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 19})">c19</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 20})">c20</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 21})">c21</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 22})">c22</button>
      <button v-on:click="emulateMessage({type: 'system.custom.event.core', custom: 23})">c23</button>
    </div>
  </div>`
})

const log = Vue.component('Log', {
  data () {
    return {
      log: null,
      list: null,
      allow: true
    }
  },
  computed: {
    messages: function() { return this.$store.getters.messages }
  },
  methods: {
    showData: function(index, message) {
      switch (message.type) {
        case 'event':
          message.childs = []
          let event = this.$store.getters.eventById(message.id)
          if (event) {
            let childActions = this.$store.getters.actions.filter(item => { return item.eventUuid === event.uuid })
            childActions.forEach(a => {
              message.childs.push({
                type : 'action',
                text : 'ACTION: ' + JSON.stringify(a)
              })
            })
            let childEvents = this.$store.getters.events.filter(item => { return item.parentUuid === event.uuid })
            childEvents.forEach(e => {
              message.childs.push({
                type : 'event',
                text : 'EVENT: ' + JSON.stringify(e)
              })
            })
            let childStates = this.$store.getters.states.filter(item => { return item.eventUuid === event.uuid })
            childStates.forEach(s => {
              message.childs.push({
                type : 'state',
                text : 'STATE: ' + JSON.stringify(s)
              })
            })
          }
          this.$store.dispatch('updateMessage', {
            index   : index,
            message : message
          })
          break;
      }
    }
  },
  mounted () {
    this.log = document.getElementById('log')
    this.list = this.log.firstElementChild
  },
  beforeUpdate () {
    let currentScroll = this.log.scrollTop
    let fullScroll = this.list.clientHeight - this.log.clientHeight

    if (fullScroll > 0 && currentScroll !== fullScroll) {
      this.allow = false
    } else {
      this.allow = true
    }
  },
  updated () {
    if (this.allow) {
      this.log.scrollTop = this.list.clientHeight - this.log.clientHeight
    }
  },
  template: `
    <div class="message-list">
      <div v-for="message, index in messages" class="message-item" v-bind:class="message.type" v-on:click="showData(index, message)">
        <div>{{ message.text }}</div>
        <div v-for="child, index in message.childs" class="child-message-item">{{ child.text }}</div>
      </div>
    </div>`
})

const app = new Vue({
  el: '#app',
  store,
  computed: {

  },
  template: `
    <div class="layout">
      <div id="main">
        <Iframe></Iframe>
        <Menu></Menu>
      </div>
      <div id="log">
        <Log></Log>
      </div>
    </div>`
})
