const https = require('https')
const express = require('express')
const ExpressSession = require('express-session')
const SharedSession = require('express-socket.io-session')
const fs = require('fs')
const moment = require('moment')
const uuidv4 = require('uuid/v4')

let RedisStore = require('connect-redis')(ExpressSession)
let privateKey = fs.readFileSync('./ssl/cert.key')
let certificate = fs.readFileSync('./ssl/cert.crt')

// ...
const useSecure = false
const port = 8099
const app = express()
const session = ExpressSession({
  store: new RedisStore({}),
  genid: function(req) {
    return uuidv4()
  },
  secret: 'emulator',
  resave: true,
  saveUninitialized: true
})

// session
app.use(session)
// static
app.use(express.static('public'))
app.get('/', function (req, res) {
  console.log('session', req.session)
  if (typeof req.session.robot === 'undefined') {
    req.session.robot = {
      session: req.sessionID
    }
  }
  res.sendFile(__dirname + '/index.html')
})
// server
const server = useSecure ? https.createServer({
  key: privateKey,
  cert: certificate,
  requestCert: false,
  rejectUnauthorized: false
}, app).listen(port) : require('http').Server(app).listen(port)
const io = require('socket.io').listen(server)

// data
const applicationData = require('./data/applicationData')
const generalImage = require('./data/generalImage')

// io
io.use(SharedSession(session))
io.on('connection', function (socket) {
  console.log('io session', socket.handshake.sessionID)
  // ...
  if (typeof socket.handshake.session.robot === 'undefined') {
    console.log('io create robot')
    socket.handshake.session.robot = {
      session: socket.handshake.sessionID
    }
  }

  // команды от эмулятора на изменение состояния робота
  socket.on('emulator-state', function (data) {
    console.log('io session', socket.handshake.sessionID)
    console.log('io robot', socket.handshake.session.robot)
  })

  // команды от эмулятора отправляем в приложение, как будто их отправил робот
  socket.on('emulator-command', function (data) {
    console.log('io session', socket.handshake.sessionID)
    console.log('io robot', socket.handshake.session.robot)
    console.log('emulator-command', data)
  })

  socket.on('api', function(data) {
    console.log('io session', socket.handshake.sessionID)
    console.log('io robot', socket.handshake.session.robot)
    console.log('api', data)
  })
})
