const https = require('https')
const express = require('express')
const session = require('express-session')
const sharedsession = require('express-socket.io-session')
const fs = require('fs')
const moment = require('moment')

let privateKey = fs.readFileSync('./ssl/cert.key')
let certificate = fs.readFileSync('./ssl/cert.crt')

// ...
const useSecure = false
const port = 8089
const app = express()

// session
app.use(session({
  secret: 'emulator',
  resave: true,
  saveUninitialized: true,
  cookie: { secure: true }
}))
// static
app.use(express.static('public'))
app.get('/', function (req, res) {
  console.log('session', req.session)
  if (typeof req.session.robot === 'undefined') {
    req.session.robot = {}
  }
  res.sendFile(__dirname + '/index.html')
})
// server
const server = useSecure ? https.createServer({
    key: privateKey,
    cert: certificate,
    requestCert: false,
    rejectUnauthorized: false
}, app).listen(port) : require('http').Server(app).listen(port)
const io = require('socket.io').listen(server)

let robotState = {
  environment: {
    id: 'promobotv4_0058',
    name: 'emulator',
    uuid_owner: '{706300c0-3e8d-473b-971a-3faea3c22a54}',
    uuid_robot: '{706300c0-3e8d-473b-971a-3faea3c22a54}'
  },
  settings: {
    system: {
      interaction: {
        startByFace: 1,
        updateByFaceInDrive: 0
      },
      timeoutInteraction: 10
    }
  },
  state: {
    faces: [],
    interaction: false,
    interactionTimer: null,
    usersDatabase: [
      {
        "age": 35,
        "gender": 1,
        "id": 4,
        "last_visit": "2019-02-22 09:00:00", // moment().add(-6, 'H'),
        "meet_date": "2018-06-15 10:24:30",
        "name": "Ходырев",
        "untouch": false,
        "extra_data": '{"workRole":"HEAD"}'
      }, {
        "age": 28,
        "gender": 1,
        "id": 5,
        "last_visit": "2019-02-22 09:00:00", // moment().add(-3, 'H'),
        "meet_date": "2018-06-15 10:24:30",
        "name": "Шабалдас",
        "untouch": false,
        "extra_data": '{"workRole":"HEAD"}'
      }
    ],
    lastQuestion: null,
    speakersVolume: 10,
    micVolume: 10,
    driveState: 'SUCCESS',
    printerStatus: 'SUCCESS',
    scannerStatus: 'SUCCESS',
    dispenserStatus: 'SUCCESS'
  }
}

// data
const applicationData = require('./data/applicationData')
const generalImage = require('./data/generalImage')

// io
// io.use(sharedsession(session, {
//     autoSave:true
// }))
io.on('connection', function (socket) {

  console.log('connection', socket.handshake.session)
  // socket.handshake.session.robot = userdata;

  // команды от эмулятора на изменение состояния робота
  socket.on('emulator-state', function(data) {

    // faces
    switch (data.meta) {
      case 'ADD-FACE':
        if (robotState.state.faces.some(face => face.id === data.data.id) === false) {
          robotState.state.faces.push(data.data)
          let info = {}
          robotState.state.faces.forEach(face => {
            info['face-' + face.id] = face
          })
          io.emit('emulator', { type: 'face.recognize.info', info: info })
          clearTimeout(robotState.state.interactionTimer)
          if (robotState.state.interaction === false) {
            robotState.state.interaction = true
            io.emit('emulator', { type: 'system.interaction.state', state: 'Interaction' })
          }
        }
        break
      case 'REMOVE-FACE':
        if (robotState.state.faces.some(face => face.id === data.data.id) === true) {
          robotState.state.faces = robotState.state.faces.filter(face => face.id !== data.data.id)
          let info = {}
          robotState.state.faces.forEach(face => {
            info['face-' + face.id] = face
          })
          io.emit('emulator', { type: 'face.recognize.info', info: info })
          if (robotState.state.interaction === true && robotState.state.faces.length === 0) {
            robotState.state.interactionTimer = setTimeout(() => {
              robotState.state.interaction = false
              io.emit('emulator', {type: 'system.interaction.state', state : 'NonInteraction'})
            }, 10000)
          }
        }
        break
    }

    if (data.meta === 'ROBOT-ID') {
      robotState.environment.id = data.data
    }
    if (data.meta === 'PRINTER-STATE') {
      robotState.printerStatus = data.data
    }
    if (data.meta === 'DISPENSER-STATE') {
      robotState.dispenserStatus = data.data
    }
    if (data.meta === 'DRIVE-STATE') {
      robotState.driveState = data.data
    }
    if (data.meta === 'SCANNER-STATE') {
      robotState.scannerStatus = data.data
    }
  })

  // команды от эмулятора отправляем в приложение, как будто их отправил робот
  socket.on('emulator-command', function(data) {
    console.log('emulator-command', data)
    io.emit('emulator', data)
    if (data.type === 'dialog.user.said') {
      console.log(data)
      if (data.text === 'проверить') {
        io.emit('emulator', { type: 'dialog.action', meta: ['check'] })
      }
    }
  })

  // логируем события из приложения
  socket.on('event', function(data) {
    // console.log('event', data)
    io.emit('log', data)
  })

  // message from application to robot api
  // получаем сообщения от приложения и эмулируем ответы робота на них
  socket.on('api', function(data) {
    if (typeof data.type !== 'undefined') {

      console.log('message to robot api', data)

      // запрос на изменение языка
      if (data.type === 'system.language.set') {

      }

      if (data.type === 'payment.terminal.request') {
        if (typeof data.state !== 'undefined') {
          io.emit('emulator', {type: 'payment.terminal.response', state: {}})
        }
        if (typeof data.sale !== 'undefined') {
          io.emit('emulator', {type: 'payment.terminal.response', sale: {}})
        }
        if (typeof data.refund !== 'undefined') {
          io.emit('emulator', {type: 'payment.terminal.response', refund: {}})
        }
      }

      // запрос по изменению получению системных настроек
      if (data.type === 'system.sound.properties') {
        // получаем громкость звука
        if (typeof data.get_volume !== 'undefined') {
          io.emit('emulator', {type: 'system.sound.load', volume: robotState.speakersVolume})
        }
        // устанавливаем громкость звука
        if (typeof data.set_volume !== 'undefined') {
          robotState.speakersVolume = data.set_volume
          // io.emit('emulator', {type: 'system.sound.load', volume: robotState.speakersVolume})
        }
        // получаем громкость микрофона
        if (typeof data.get_mic !== 'undefined') {
          io.emit('emulator', {type: 'system.sound.load', volume: robotState.micVolume})
        }
        // устанавливаем громкость микрофона
        if (typeof data.set_mic !== 'undefined') {
          robotState.micVolume = data.set_mic
          // io.emit('emulator', {type: 'system.sound.load', volume: robotState.micVolume})
        }
      }

      // запрос по работе с пользователями
      if (data.type === 'database.user.request') {
        // добавление пользователя
        if (typeof data.add_user !== 'undefined') {
          let index = robotState.state.usersDatabase.findIndex(u => u.id === data.add_user.id)
          if (index === -1) {
            robotState.state.usersDatabase.push(data.add_user)
            // io.emit('emulator', {type: 'database.user.response', response: {add_user: true}})
          } else {
            // io.emit('emulator', {type: 'database.user.response', response: {add_user: false}})
          }
          // пока так, метод в любом случае отвечает true
          io.emit('emulator', {type: 'database.user.response', response: {add_user: true}})
        }
        // удаление пользователя
        if (typeof data.del_user !== 'undefined') {
          let index = robotState.state.usersDatabase.findIndex(u => u.id === data.del_user)
          if (index !== -1) {
            robotState.state.usersDatabase.splice(index, 1)
            io.emit('emulator', {type: 'database.user.response', response: {del_user: true}})
          } else {
            io.emit('emulator', {type: 'database.user.response', response: {del_user: false}})
          }
        }
        // получение пользователя
        if (typeof data.get_user !== 'undefined') {
          let index = robotState.state.usersDatabase.findIndex(u => u.id === data.get_user)
          if (index !== -1) {
            io.emit('emulator', {type: 'database.user.response', response: {get_user: robotState.state.usersDatabase[index]}})
          } else {
            io.emit('emulator', {type: 'database.user.response', response: {get_user: false}})
          }
        }
      }

      // интерактивное взаимодействие
      if (data.type === 'system.interaction.control') {
        switch (data.state) {
          case 'Start':
            // приложение сообщило о начале взаимодействия, отправляем ответ от api робота
            if (robotState.state.interaction === false) {
              robotState.state.interaction = true
              io.emit('emulator', {type: 'system.interaction.state', state : 'Interaction'})
            }
            break;
          case 'Stop':
            // приложение сообщило об окончании взаимодействия, отправляем ответ от api робота
            if (robotState.state.interaction === true) {
              robotState.state.interaction = false
              io.emit('emulator', {type: 'system.interaction.state', state : 'NonInteraction'})
            }
            break;
        }
      }

      // эмулируем получение настроек от робота
      if (data.type === 'system.settings.get') {
        let settings = {
          s1: {
            settings_type: data.settings_type,
            group: data.group,
            value: 10
          }
        }
        if (typeof data.name !== 'undefined') {
          settings.s1.name = data.name
        }
        if (data.group === 'dialog' && data.name.timeoutResetAnchor) {
          settings = {
            s1: {
              settings_type: data.settings_type,
              group: data.group,
              name: data.name,
              value: 20
            }
          }
        }
        if (data.group === 'mail') {
          let s = {}
          if (typeof data.name !== 'undefined' && data.name === 'to') {
            s['s1'] = {
              settings_type: data.settings_type,
              group: data.group,
              name: data.name,
              value: 'a@okolovich.com'
            }
          }
          if (typeof data.name !== 'undefined' && data.name === 'subject') {
            s['s2'] = {
              settings_type: data.settings_type,
              group: data.group,
              name: data.name,
              value: 'subject'
            }
          }
          if (typeof data.name !== 'undefined' && data.name === 'body') {
            s['s3'] = {
              settings_type: data.settings_type,
              group: data.group,
              name: data.name,
              value: 'text {text} text'
            }
          }
          settings = s
        }
        if (data.group === 'js') {
          settings = {
            s1: {
              settings_type: data.settings_type,
              group: data.group,
              name: 'debug',
              value: true
            },
            s2: {
              settings_type: data.settings_type,
              group: data.group,
              name: 'debugUrl',
              value: ((useSecure) ? 'https:' : 'http:') + '//localhost:8089'
            }
          }
        }
        if (data.group === 'headers') {
          settings = {
            s1: {
              settings_type: data.settings_type,
              group: data.group,
              name: 'h01',
              value: 'h01'
            },
            s2: {
              settings_type: data.settings_type,
              group: data.group,
              name: 'h02',
              value: 'h02'
            }
          }
        }
        // ...
        setTimeout(() => {
          io.emit('emulator', {
            type: 'system.settings.load',
            id: null,
            settings: settings
          })
        }, Math.floor(Math.random() * (1600 - 200)) + 200)
      }

      // эмулируем получение окружения робота
      if (data.type === 'system.environment.get') {
        io.emit('emulator', {type: 'system.environment.load', environment: robotState.environment})
      }

      // при команде на печать, сохраняем картинкой полученные данные в base64
      if (data.type === 'printer.print') {
        try {
          let base64Data = data.data.replace(/^data:image\/png;base64,/, "")
          fs.writeFile('./printer/' + moment().format() + '.png', base64Data, 'base64', function(err) {
            console.log(err)
          })
        } catch (e) {
          console.log(e)
        }
        if (robotState.printerStatus === 'ERROR') {
          io.emit('emulator', {type: 'printer.error'})
        }
        if (robotState.printerStatus === 'SUCCESS') {
          io.emit('emulator', {type: 'printer.success'})
        }
      }

      // отвечаем на запрос статуса принтера
      if (data.type === 'printer.state') {
        if (robotState.printerStatus === 'ERROR') {
          io.emit('emulator', {type: 'printer.state.info', state: 'ERROR'})
        }
        if (robotState.printerStatus === 'SUCCESS') {
          io.emit('emulator', {type: 'printer.state.info', state: 'IDLE'})
        }
      }

      // работа с диспенсером
      if (data.type === 'dispenser.dispense') {
        // по команде выдать карту высылаем набор статусы диспенсера
        // смотрим на заданное состояние диспенсера
        if (robotState.dispenserStatus === 'NOT-CONNECTED') {
          io.emit('emulator', {type: 'dispenser.not.connected'})
        }
        if (robotState.dispenserStatus === 'ERROR') {
          io.emit('emulator', {type: 'dispenser.card.error'})
        }
        if (robotState.dispenserStatus === 'SUCCESS') {
          io.emit('emulator', {type: 'dispenser.card.dispensing'})
          io.emit('emulator', {type: 'dispenser.dispense'})
        }
        // io.emit('emulator', {type: 'dispenser.communication.timeout'})
        // io.emit('emulator', {type: 'dispenser.card.preempty'})
        // io.emit('emulator', {type: 'dispenser.card.empty'})
        // io.emit('emulator', {type: 'dispenser.cardbin.full'})
        // io.emit('emulator', {type: 'dispenser.nodispenser'})
      }

      // команды по управлению движением
      if (data.type === 'system.drive.control') {
        // команда отправки на точку
        if (typeof data.drive_point !== 'undefined') {
          // отправляем сообщение о том что робот поехал
          io.emit('emulator', {type: 'system.drive.status', status : 1})
          setTimeout(() => {
            // отправляем сообщение о том что робот приехал
            switch (robotState.driveState) {
              case 'SUCCESS':
                io.emit('emulator', {type: 'system.drive.status', status: 2})
                break
              case 'STUCKED':
                io.emit('emulator', {type: 'system.drive.status', status: 3})
                setTimeout(() => {
                  io.emit('emulator', {type: 'system.drive.status', status: 3})
                }, 1000)
                setTimeout(() => {
                  io.emit('emulator', {type: 'system.drive.status', status: 3})
                }, 2000)
                setTimeout(() => {
                  io.emit('emulator', {type: 'system.drive.status', status: 3})
                }, 3000)
                break
              case 'OBSTACLE':
                io.emit('emulator', {type: 'system.drive.status', status: 4})
                break
            }
          }, 8000)
        }
        // команда отправки на зарядное устройство
        if (typeof data.drive_charger !== 'undefined') {
          // сообщаем что робот поехал
          io.emit('emulator', {type: 'system.drive.status', status: 1})
          setTimeout(() => {
            // отправляем сообщение о том что робот приехал
            switch (robotState.state.driveState) {
              case 'SUCCESS':
                io.emit('emulator', {type: 'system.drive.charge', charge: 0})
                break
            }
          }, 5000)
        }
        // команда изменения режима
        if (typeof data.drive_mode !== 'undefined') {
          switch (data.drive_mode) {
            case 0:
              // сообщаем что режим движения сменился на ручной
              io.emit('emulator', {type: 'system.drive.mode', mode: 0})
              break;
            case 1:
              // сообщаем что режим движения сменился на авто
              io.emit('emulator', {type: 'system.drive.mode', mode: 1})
              break;
          }
        }
      }

      // получение реплики из лингвобазы
      if (data.type === 'dialog.replica.get') {
        // получение по типу реплики
        if (typeof data.replica_type !== 'undefined') {
          switch (data.replica_type) {
            case 'GREETING':
              io.emit('emulator', {type: 'dialog.replica.load', replica: {text: 'GREETING'}})
              break;
          }
        }
        // получение по названию шага
        if (typeof data.step !== 'undefined') {
          switch (data.step) {
            case 'test_step':
              io.emit('emulator', {type: 'dialog.replica.load', replica : {text: 'Тестовое приложение'}})
              break;
            default:
              if (typeof applicationData.steps !== 'undefined') {
                let step = applicationData.steps.find(s => s.name === data.step)
                if (step && step.replica) {
                  io.emit('emulator', {
                    type: 'dialog.replica.load',
                    replica: step.replica
                  })
                }
              }
              break;
          }
        }
      }

      // команда роботу произнести реплику
      if (data.type === 'dialog.say') {
        if (robotState.interaction === false) {
          robotState.interaction = true
        }
        if (typeof data.step !== 'undefined') {
          let replicFinishTimeout = 4 * 1000
          switch (data.step) {
            case 'test_step':
              // отправляем сообщение о том что начали произносить текст
              io.emit('emulator', {type: 'dialog.robot.replic.start', text: 'Тестовое приложение'})
              break;
            default:
              if (typeof applicationData.steps !== 'undefined') {
                let step = applicationData.steps.find(s => s.name === data.step)
                if (step && step.messages) {
                  step.messages.forEach(message => {
                    // фиксируем последний заданный вопрос
                    if (message.type === 'dialog.question.new') {
                      robotState.state.lastQuestion = step
                    }
                    if (message.type === 'dialog.robot.replic.start') {
                      replicFinishTimeout = message.text.length * 0.12 * 1000
                    }
                    io.emit('emulator', message)
                  })
                }
              }
              break;
          }
          // отправляем сообщение что перестали произносить реплику
          setTimeout(() => {
            io.emit('emulator', {type: 'dialog.robot.replic.finish'})
          }, replicFinishTimeout)
        }
        if (typeof data.text !== 'undefined') {
          io.emit('emulator', {type: 'dialog.robot.replic.start', text: data.text})
          setTimeout(() => {
            io.emit('emulator', {type: 'dialog.robot.replic.finish'})
          }, 2000)
        }
        if (typeof data.replica !== 'undefined') {
          if (typeof data.replica.id !== 'undefined') {
            // ...
            let step = applicationData.steps.find(s => (typeof s.replica !== 'undefined') && s.replica.id === data.replica.id)
            if (step) {
              if (step && step.messages) {
                step.messages.forEach(message => {
                  // фиксируем последний заданный вопрос
                  if (message.type === 'dialog.question.new') {
                    robotState.state.lastQuestion = step
                  }
                  if (message.type === 'dialog.robot.replic.start') {
                    replicFinishTimeout = message.text.length * 0.12 * 1000
                  }
                  io.emit('emulator', message)
                })
              }
            } else if (typeof data.replica.text !== 'undefined') {
              io.emit('emulator', {type: 'dialog.robot.replic.start', text: data.replica.text})
            }
          }
        }
      }

      // сообщение из приложения что пользователь выбрал ответ на экране
      // смотрим ответы из последнего заданного вопроса
      if (data.type === 'dialog.question.answer') {
        if (typeof data.text !== 'undefined') {
          if (robotState.state.lastQuestion !== null) {
            // есть заданный вопрос
            let answer = robotState.state.lastQuestion.answers.find(a => a.text === data.text)
            if (answer && answer.messages) {
              answer.messages.forEach(message => {
                io.emit('emulator', message)
              })
            }
          } else {
            // правило
          }
        }
      }

      // faces
      if (data.type === 'face.recognize.control') {
        // head_enabled
        // tracking_auto
        if (typeof data.tracking_face !== 'undefined') {
          io.emit('emulator', {type: 'system.general.setted', setted_id: 1, id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'})
        }
        if (typeof data.get_general_frame !== 'undefined') {
          io.emit('emulator', {type: 'face.general.frame', image: generalImage, id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'})
        }
      }

      if (data.type === 'system.general.touch') {
        if (data.x <= 0.5) {
          io.emit('emulator', {type: 'system.general.setted', setted_id: 1, id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'})
        } else {
          io.emit('emulator', {type: 'system.general.setted', setted_id: 2, id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'})
        }
      }

      // ...
      if (data.type === 'mail.sender.request') {
        setTimeout(() => {
          io.emit('emulator', {type: 'mail.sender.response', status: 'success', id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'})
        }, 2000)
      }

      // сканер документов
      if (data.type === 'scanner.document.request') {
        let scanTime = 4000
        // ...
        if (robotState.scannerStatus === 'ERROR') {
          io.emit('emulator', {type: 'scanner.document.response', success: true, id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'})
          io.emit('emulator', {type: 'scanner.document.state', state: 1, id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'})
          for (let i = 0; i < 20; i++) {
            setTimeout(() => {
              io.emit('emulator', {type: 'scanner.document.progress', progress: 0, id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'})
            }, (scanTime / 20 * i))
          }
          setTimeout(() => {
            io.emit('emulator', {
              type: 'scanner.document.finish', data: {
                timeoutProcess: 'true'
              },
              id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'
            })
          }, scanTime + 200)
        } else {
          io.emit('emulator', {type: 'scanner.document.response', success: true, id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'})
          io.emit('emulator', {type: 'scanner.document.state', state: 1, id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'})
          for (let i = 0; i < 20; i++) {
            setTimeout(() => {
              io.emit('emulator', {type: 'scanner.document.progress', progress: (100 / 20 * i), id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'})
            }, (scanTime / 20 * i))
          }
          setTimeout(() => {
            io.emit('emulator', {
              type: 'scanner.document.finish', data: {
                outputFileName: '/home/file_25:09:2018_11:03:11.456.pnm'
              },
              id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'
            })
          }, scanTime + 200)
          setTimeout(() => {
            io.emit('emulator', {type: 'scanner.document.state', state: 0, id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'})
          }, scanTime + 400)
        }
      }

      // эмулируем sip
      if (data.type === 'sip.control') {
        if (typeof data.call_start !== 'undefined') {
          io.emit('emulator', {type: 'sip.call.response', result: true, id: '{eb4fbdc3-4f8b-46bd-aee2-b8f3cbfd044e}'})
        }
        if (typeof data.call_stop !== 'undefined') {
          // ...
        }
        if (typeof data.play_text !== 'undefined') {
          io.emit('emulator', {type: 'sip.text.response'})
        }
        if (typeof data.play_file !== 'undefined') {
          io.emit('emulator', {type: 'sip.file.response'})
        }
        if (typeof data.clear_queue !== 'undefined') {
          // ...
        }
      }
      // ...
    }
 })
});
